/* Read input files */

	proc import datafile = 'Q:\Business_Plans\2023BP\Electric\3_Data_Processing\1_Billed_to_Calendar\Analysis\SASInputs_Allocators.csv' out = inp_Allocators DBMS = CSV REPLACE;
	run;

	proc import datafile = 'Q:\Business_Plans\2023BP\Electric\3_Data_Processing\1_Billed_to_Calendar\Analysis\SASInputs_IndVarFcsts_AVG.csv' out = inp_IndVars DBMS = CSV REPLACE;
	run;

	proc import datafile = 'Q:\Business_Plans\2023BP\Electric\3_Data_Processing\1_Billed_to_Calendar\Analysis\SASInputs_BilledCoeff.csv' out = inp_BilledCoeff DBMS = CSV REPLACE;
	run;

	proc import datafile = 'Q:\Business_Plans\2023BP\Electric\3_Data_Processing\1_Billed_to_Calendar\Analysis\SASInputs_BilledFcsts.csv' out = inp_BilledFcsts DBMS = CSV REPLACE;
	run;


/* The Billed-to-Calendar process is completed in five steps.  Forecasts comprise one or more "components."  Each component is allocated to calendar months based on the allocation of 
	either billed DDs or billing days to calendar months.  */

/* Step 1.  Admin... */

	/* Create copy of inp_IndVars to separately reference forecast-to-rate allocation ratios ("RateRatio"). */
	data inp_IndVars_copy;
		set inp_IndVars;
	run;

	/* Create table to map forecasts to rates. */
	proc sql;
		Create table ratemap as
			select a.forecast, a.fcst_model, a.rate, count(a.rate) as cnt from inp_billedfcsts as a group by a.forecast, a.fcst_model, a.rate;
	quit;

/* Step 2.  For forecast components that are included in this process (Include in inp_BilledCoeff = 1), calculate components where the coefficient 
	applies to ALL months (Bill_Month in inp_BilledCoeff = 0) and allocate to calendar months. */

	proc sql;
		create table step1 as 
			select a.co, a.fcst_model, a.forecast, a.RateRatio, a.Indvar_name, a.coeff, a.Include, b.billyear, b.billmonth, c.calyear, c.calmonth, b.value, c.ratio, c.offset, b.alloc_type, d.value as rateratio_val, 
				a.coeff*b.value*d.value as bill_component, a.coeff*b.value*d.value*c.ratio as cal_component
				from ((inp_billedcoeff as a inner join inp_Indvars as b on (a.co = b.co and a.Indvar_name = b.Indvar_name)) 
					inner join inp_allocators as c on (b.co = c.co and b.alloc_type = c.alloc_type and b.billyear = c.billyear and b.billmonth = c.billmonth))
					inner join inp_IndVars_copy as d on (a.co = d.co and a.RateRatio = d.Indvar_name and b.billyear = d.billyear and b.billmonth = d.billmonth)
				where a.billmonth = 0 and a.Include = 1 /* and a.forecast = "LG_FCGS_NONMA" and b.billyear = 2022 and b.billmonth = 12 */
				order by a.co, a.forecast, a.Indvar_name, b.billyear, b.billmonth;
	quit;

/* Step 3.  For forecast components that are included in this process (Include in inp_BilledCoeff = 1), calculate components where the coefficient  
	applies only to SPECIFIC months (Bill_Month in inp_BilledCoeff NE 0) and allocate to calendar months.  */

	proc sql;
		create table step2 as 
			select a.co, a.fcst_model, a.forecast, a.RateRatio, a.Indvar_name, a.coeff, a.Include, b.billyear, b.billmonth, c.calyear, c.calmonth, b.value, c.ratio, c.offset, b.alloc_type, d.value as rateratio_val,
				a.coeff*b.value*d.value as bill_component, a.coeff*b.value*d.value*c.ratio as cal_component
				from ((inp_billedcoeff as a inner join inp_Indvars as b on (a.co = b.co and a.Indvar_name = b.Indvar_name and a.billmonth = b.billmonth)) 
					inner join inp_allocators as c on (b.co = c.co and b.alloc_type = c.alloc_type and b.billyear = c.billyear and b.billmonth = c.billmonth))
					inner join inp_IndVars_copy as d on (a.co = d.co and a.RateRatio = d.Indvar_name and b.billyear = d.billyear and b.billmonth = d.billmonth)
				where a.billmonth NE 0 and a.Include = 1 /* and a.forecast = "LG_FCGS_NONMA" and b.billyear = 2022 and b.billmonth = 12 */
				order by a.co, a.forecast, a.Indvar_name, b.billyear, b.billmonth;
	quit;

	/* Append results of step2 to step1. */
	data temp;
		set step1 step2;
		if billmonth ne calmonth then bill_component = 0;
	run;

/* Step 4.  CHECK billed results to ensure billed forecasts from this multiplication process match billed forecasts in billed forecast summary workbook.  
		If the compare_billed table has differences and the source of the problem is not obvious, the temp table (created immediately above) contains 
		"BX" results for detailed inspection.  */

	proc sql;
		create table billed_fcsts as
			select a.billyear, a.billmonth as month, a.co, a.forecast, sum(a.value) as correctvalue
				from inp_billedfcsts as a
				group by a.co, a.forecast, a.billyear, a.billmonth
				order by a.co, a.forecast, a.billyear, a.billmonth;
	quit;

	proc sql;
		Create table billed_b2c_check as
			select a.billyear, a.billmonth as month, a.co, a.forecast, b.rate, sum(a.bill_component) as value, "billed" as type
				from temp as a inner join ratemap as b on a.forecast = b.forecast
				group by a.co, a.forecast, b.rate, a.billyear, a.billmonth;
	run;

	proc sql;
		create table compare_billed as
			select a.*, b.correctvalue, (a.value - b.correctvalue) as Diff
				from billed_b2c_check as a inner join billed_fcsts as b
				on a.co = b.co and a.billyear = b.billyear and a.month = b.month and a.forecast = b.forecast;
	quit;

/* Step 5.  Summarize results.  */

	proc sql;
		Create table billed_b2c as
			select a.billyear as year, a.billmonth as month, a.co, a.forecast, a.fcst_model, b.rate, sum(a.bill_component) as value, "billed" as type
				from temp as a inner join ratemap as b on a.forecast = b.forecast
				group by a.co, a.forecast, a.fcst_model, b.rate, a.billyear, a.billmonth;
	run;

	proc sql;
		Create table calendar_b2c as
			select a.calyear as year, a.calmonth as month, a.co, a.forecast, a.fcst_model, b.rate, sum(a.cal_component) as value, "calendar" as type
				from temp as a inner join ratemap as b on a.forecast = b.forecast
				group by a.co, a.forecast, a.fcst_model, b.rate, a.calyear, a.calmonth;
	run;

	proc sql;
		Create table unbilled_b2c as
			select a.calyear as year, a.calmonth as month, a.co, a.forecast, a.fcst_model, b.rate, sum(a.cal_component) as value, "unbilled" as type
				from temp as a inner join ratemap as b on a.forecast = b.forecast
				where a.offset LT 0
				group by a.co, a.forecast, a.fcst_model, b.rate, a.calyear, a.calmonth;
	run;

	data results;
		set calendar_b2c billed_b2c unbilled_b2c;
	run;




